Decay

Enum Decay 

pub enum Decay {
    None(f32),
    Linear {
        start: f32,
        decay: f32,
    },
    Cosine {
        start: f32,
        end: f32,
        num_steps: usize,
    },
}
A value which can decay according to some schedule over the progression of timesteps.

Variants



None(f32)

No decay: The specified constant is used regardless of timestep.



Linear

Linear decay: The value decays from start by decay each timestep.

Fields

start: f32
decay: f32


Cosine

Cosine decay: The value smoothly decays from start to end over num_steps timesteps, with the decay matching the shape of the cosine function.

Fields

start: f32
end: f32
num_steps: usize

Implementations

Source

impl Decay

pub fn start_value(&self) -> f32

The output value at timestep 0.

pub fn at_timestep(&self, timestep: usize) -> f32

The output value at the specified timestep.