ExpAvg

Struct ExpAvg 

pub struct ExpAvg { /* private fields */ }
Computes a rolling exponential average, based on the given alpha value.

An alpha of 0.5 to 1.0 gives higher weight to the most recent sample, whereas an alpha of 0.1 to 0.3 gives higher weight to past samples.

Implementations

Source

impl ExpAvg

pub fn new(alpha: f32) -> Self

Creates a new ExpAvg with the given smoothing factor.

pub fn update(&mut self, sample: f32) -> f32

Updates the moving average with a new sample.

pub fn get(&self) -> Option<f32>

Returns the current average.