pub struct TrainInfo {
pub lr: f32,
pub l1_reg: f32,
pub l2_reg: f32,
pub grad_clip: Option<f32>,
pub step: usize,
}Expand description
Describes the training parameters at some instant, serialized during recording.
Fields
lr: f32The learning rate used this update step.
l1_reg: f32The L1 regularization applied each update step.
l2_reg: f32The L2 regularization applied each update step.
grad_clip: Option<f32>The maximum magnitude of any update to any parameter.
step: usizeThe 0-indexed count of update steps.