GradAdjuster

Trait GradAdjuster 

pub trait GradAdjuster<G: Gradients> {
    // Required method
    fn adjust(&mut self, gradient_updates: G, loss: f32) -> G;
}
An object responsible for tweaking gradient updates, such as to add the effects of momentum, perform gradient clipping, etc.

Required Methods

fn adjust(&mut self, gradient_updates: G, loss: f32) -> G

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors