pub trait Module<X> {
type Output;
// Required method
fn forward(&self, x: &X) -> Result<Self::Output, Error>;
}Expand description
A unit of computation that consumes Input and produces Module::Output.
pub trait Module<X> {
type Output;
// Required method
fn forward(&self, x: &X) -> Result<Self::Output, Error>;
}A unit of computation that consumes Input and produces Module::Output.