pub enum Decay {
None(f32),
Linear {
start: f32,
decay: f32,
},
Cosine {
start: f32,
end: f32,
num_steps: usize,
},
}Expand description
A value which can decay according to some schedule over the progression of timesteps.
Variants
None(f32)
No decay: The specified constant is used regardless of timestep.
Linear
Linear decay: The value decays from start by decay each timestep.
Cosine
Cosine decay: The value smoothly decays from start to end over num_steps timesteps,
with the decay matching the shape of the cosine function.
Implementations
Sourceimpl Decay
impl Decay
Sourcepub fn start_value(&self) -> f32
pub fn start_value(&self) -> f32
The output value at timestep 0.
Sourcepub fn at_timestep(&self, timestep: usize) -> f32
pub fn at_timestep(&self, timestep: usize) -> f32
The output value at the specified timestep.