pub struct ExpAvg { /* private fields */ }Expand description
Computes a rolling exponential average, based on the given alpha value.
An alpha of 0.5 to 1.0 gives higher weight to the most recent sample, whereas an alpha of 0.1 to 0.3 gives higher weight to past samples.
Implementations